<? 
/*
* phorSale - An eCommerce application for PHP and mySQL.
* Copyright (C) 2001 Michael Kmiec <mike@shifter.org>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/*****
*
*   CyberCashWrapper:
*
*       1. Build the URL to send to CyberCash CGI
*       2. Parse response from CGI
*       3. Return appropriate information
*
*       The CGI handles all Merchant information (CCID, etc) 
*       so it isn't a concern here. The CGI passes back a pipe ("|")
*       delimited string which this class parses. To see how 
*       this works, refer to "CyberCash.cgi" in this distribution.
*
*****/
  
class CyberCashWrapper
{
    var $orderID;
    var $order = array();
    var $ccInfo = array();
    var $server = "http://YOUR_DOMAIN_HERE/cgi-bin/CyberCash.cgi";
    var $queryString;
    var $responseString;
    var $response = array();
    var $isDeclined = false;
    var $isExpired = false;

    //*****
    // Constructor
    function CyberCashWrapper( $oID, $o, $cci )
    { 
        $this->orderID = $oID;
        $this->order = $o;
        $this->ccInfo = $cci;

        // now build the appropriate query string
        $this->buildQueryString();
    } 

    // *****
    // Build the url for the cgi. N.B. - This will only handle CyberCash Cash Register transactions.
    // If you want to add in Fraud Patrol service, you will have to add the variables yourself.
    function buildQueryString()
    {
        $processURL = "mo.order-id=" . urlencode($this->orderID);
        $processURL .= "&mo.price=" . urlencode( "usd " . $this->order["final_cost"]);
        
        // for testing, see "Sample Transaction Amounts, Response Codes and Messages" 
        // in the MCK User Guide for details, then reset this amount.
        //$processURL .= "&mo.price=" . urlencode( "usd 1400");

        $processURL .= "&cpi.card-number=" . urlencode($this->ccInfo["cc_number"]);
        $processURL .= "&cpi.card-exp=" . urlencode($this->ccInfo["cc_exp_m"] . "/" . $this->ccInfo["cc_exp_y"]);        
        $processURL .= "&cpi.card-type=" . urlencode($this->ccInfo["cc_type"]);

        $processURL .= "&cpi.card-name=" . urlencode($this->order["bill_name"]);
        $processURL .= "&cpi.card-address=" . urlencode($this->order["bill_addr_1"]);
        if ( strlen( trim( $this->order["bill_addr_2"] ) ))
            $processURL .= urlencode(" " . $this->order["bill_addr_2"]);

        $processURL .= "&cpi.card-city=" . urlencode($this->order["bill_city"]);
        $processURL .= "&cpi.card-state=" . urlencode($this->order["bill_state"]);
        $processURL .= "&cpi.card-zip=" . urlencode($this->order["bill_postal_code"]);
        $processURL .= (strlen(trim($this->order["bill_country"]))) ? "&cpi.card-country=" . urlencode($this->order["bill_country"]) : "&cpi.card-country=US"; 

        $this->queryString = $processURL;
        return true;
    }

    // *****
    // Call the URL of the local CGI, which in turn contacts the CyberCash server; store the response
    function getResponse()
    {
        $fp = @fopen("$this->server?$this->queryString", 'r'); 

        if( $fp == false )
        { 
            $this->response["pop.status"] = "failure-hard"; 
            $this->response["pop.error-message"] = "HTTP request failed";
            return false;
        } 

        while( $buffer = @fgets($fp, 4096) )
        {
            if ( strstr( $buffer, "|" ) )
                $this->responseString .= $buffer;
        }

        @fclose( $fp );

        $resArr = explode( "|", $this->responseString );
        for ( $i = 0; $i < count( $resArr ); $i++ )
        {
            $splitStr = $resArr[$i];
            $splitArr = split( "=", $splitStr, 2 );

            if ( strlen( $splitArr[0] ) )
                $this->response[$splitArr[0]] = trim($splitArr[1]);
        }

        return true;
    }

    // *****
    // Return the status code from this transaction
    function getStatus()
    {
        if ( stristr( $this->response["pop.status"], "success" ) )
            return true;

        if ( stristr( $this->response["pop.status"], "failure-bad-money" ) )
            $this->isDeclined = true;

        if ( $this->response["pop.action-code"] == 101 )
            $this->isExpired = true;

        return false;
    }

    // *****
    // Return the raw error code and message, if any.
    function getRawErrorMessage()
    {        
        if( !$this->getStatus() )
            return "Error " . $this->response["pop.error-code"] . ": " . $this->response["pop.error-message"];

        return "";
    }

    // *****
    // Go through the action codes sent from processor bank; in this case, NOVA
    function getMessage()
    {    
        $retstr = "";
        if( !$this->getStatus() )
        {            
            switch( $this->response["pop.action-code"] )
            {
                case "000":
                    $retstr = "Authorization approved.";
                    break;
                case "007":
                    $retstr = "Authorization approved and captured.";
                    break;
                case "100":
                    $retstr = "Declined.";
                    break;
                case "101":
                    $retstr = "Expired card.";
                    break;
                case "107":
                    $retstr = "Declined, bad card; call card issuer.";
                    break;
                case "108":
                    $retstr = "Bank/processor data source down.";
                    break;
                case "109":
                    $retstr = "Invalid merchant ID. Please contact CyberCash to resolve this problem.";
                    break;
                case "110":
                    $retstr = "Invalid amount. Please contact CyberCash to resolve this problem.";
                    break;
                case "111":
                    $retstr = "Invalid card number.";
                    break;
                case "114":
                    $retstr = "Transaction not in batch.";
                    break;
                case "115":
                    $retstr = "Format error. Please contact CyberCash to resolve this problem.";
                    break;
                case "120":
                    $retstr = "Transaction not permitted. Please contact CyberCash to resolve this problem.";
                    break;
                case "200":
                    $retstr = "Declined-pick up card.";
                    break;
                case "500":
                    $retstr = "Approved.";
                    break;                
                case "580":
                    $retstr = "Error";
                    break;
                case "581":
                    $retstr = "Out of balance.";
                    break;
                case "909":
                    $retstr = "Application problem. Please contact CyberCash to resolve this problem.";
                    break;
                default:
                    $retstr = $this->getRawErrorMessage();
                    break;
            }
        }

        return $retstr;
    }

    // *****
    // Function that iterates through the response array; for debugging.
    function viewResponse()
    {
        while (list($k, $v) = each ( $this->response ))         
            echo( "KEY: $k, VAL: $v<br>" );
    }
}
?>