<?
/*
* phorSale - An eCommerce application for PHP and mySQL.
* Copyright (C) 2001 Michael Kmiec <mike@shifter.org>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

    // ********************************************************************************************
    // class Shipment:
    //          Base class for shipping methods where shipping information
    //          is retrieved via HTTP. 
    //          
    //          Many pseudo-virtual methods in this class; inherit as necessary.
    // ********************************************************************************************
    class Shipment
    {
        var $keys;
        var $vals;
        var $msg;
        var $retcode;
        var $maxWeight;
        var $server;
        var $queryString;

        function Shipment()
        {            
            $this->msg = "";
            $this->retcode = false;
        }

        function success()
        {
            return $this->retcode;
        }

        function getMessage()
        {
            return $this->msg;
        }

        function getServer()
        {
            return $this->server;
        }

        function getQueryString()
        {
            return $this->queryString;
        }        

        function getKeys() 
        {            
            // returns an array
            return $this->keys;
        }

        function countKeys()
        {            
            return count( $this->keys );
        }

        // *****
        // Get a given value based on a hash-like structure
        function getValue( $key )
        {        
            for ( $i = 0; $i < $this->countKeys(); $i++ )
            {
                if ( strcmp( $this->keys[$i], $key ) == 0 )
                    return $this->vals[$i];
            }

            return null;
        }

        function getResponse()
        {
            // virtual
        }

        function buildQueryString()
        {
            // virtual            
        }
        
        
        function setStatus()
        {
            // virtual            
        }

        
        function setMessage()
        {
            // virtual            
        }

        
        function setPrice()
        {
            // virtual            
        }

        
        function currentServiceType()
        {
            // virtual            
        }
        
    } // end of class    


    // ********************************************************************************************
    // class UPSShipment extends Shipment:
    //          Class for UPS shipping prices and information.
    // ********************************************************************************************
    class UPSShipment extends Shipment
    {
        var $nda;
        var $sda;
        var $tds;
        var $gro;
        var $parent;

        var $NEXT_DAY = "1DA";
        var $SECOND_DAY = "2DA";
        var $THIRD_DAY = "3DS";
        var $GROUND = "GND";


        function UPSShipment()
        {
            $this->parent = new Shipment();
            $this->server = "http://www.ups.com:80/using/services/rave/qcost_dss.cgi?";            

            $this->keys = array(  "UPSOnLine",
                            "AppVersion",
                            "ReturnCode",
                            "MessageText",
                            "ActionCode",  
                            "ServiceLevelCode",
                            "ShipperPostalCode",
                            "ShipperCountry",
                            "ConsigneePostalCode", 
                            "ConsigneeCountry",
                            "DeliveryZone",
                            "PackageActualWeight",
                            "ProductCharge",
                            "AccessorySurcharge",
                            "TotalCharge",
                            "CommitTime" );
            $this->nda = 0;
            $this->sda = 0;
            $this->tds = 0;
            $this->gro = 0;
            $this->maxWeight = 150;
            $this->msg = "<br>Unable to connect to the UPS server at " . $this->server . "<br>";
        }

        // *****
        // Set the status
        function setStatus( $get )
        {
            $retval = ( $this->getValue( $get ) == "0000" ) ? true : false;
            return $retval;
        }

        function getNextDayAirPrice()
        {
            return sprintf( "%01.2f", $this->nda );
        }

        function getSecondDayAirPrice()
        {
            return sprintf( "%01.2f", $this->sda );            
        }

        function getThreeDaySelectPrice()
        {
            return sprintf( "%01.2f", $this->tds );
        }
        
        function getGroundPrice()
        {
            return sprintf( "%01.2f", $this->gro );
        }                

        // *****
        // Set a message, if applicable
        function setMessage( $get )
        {
            return $this->getValue( $get );
        }

        // *****
        // Set the price
        function setPrice( $get )
        {
            return $this->getValue( $get );
        }

        // *****
        // Return the service type for the current line
        function currentServiceType( $get )
        {
            return $this->getValue( $get );
        }

        function loadVals( $valLine, $delim = "%" )
        {
            $this->vals = array();                        
            $tok = strtok( $valLine, $delim );

            while( $tok ) 
            {
                $this->vals[count($this->vals)] = $tok;
                $tok = strtok( $delim );                
            }

            if ( empty( $this->retcode ) )
            {
                $this->retcode = $this->setStatus( "ReturnCode" );
                $this->msg = $this->setMessage( "MessageText" );    
            }

            if ( $this->success() )
            {
                if ( $this->currentServiceType("ServiceLevelCode") == $this->NEXT_DAY )
                    $this->nda += $this->setPrice("TotalCharge");
                if ( $this->currentServiceType("ServiceLevelCode") == $this->SECOND_DAY )
                    $this->sda += $this->setPrice("TotalCharge");
                if ( $this->currentServiceType("ServiceLevelCode") == $this->THIRD_DAY )
                    $this->tds += $this->setPrice("TotalCharge");
                if ( $this->currentServiceType("ServiceLevelCode") == $this->GROUND )
                    $this->gro += $this->setPrice("TotalCharge");
            }
        }

        function buildQueryString( $toCode, $fromCode, $weight, $resOrInd = "0", $toCountry = "US", $fromCountry = "US" )
        {
            // need to pass the application information
            $this->queryString = "AppVersion=1.2&AcceptUPSLicenseAgreement=YES&";

            // "ActionCode" 3 specifies a rate for a selected service; 4 specifies rates for all available 
            $this->queryString .= "ResponseType=application/x-ups-rss&ActionCode=4&";

            // "ServiceLevelCode" specifies which type of service available, even though we will get all (specified above)
            $this->queryString .= "ServiceLevelCode=GND&RateChart=Regular+Daily+Pickup&";

            // the "To" and "From" ZIP codes
            $this->queryString .= "ShipperPostalCode=$fromCode&ConsigneePostalCode=$toCode&";
        
            // "To" and "From" countries
            $this->queryString .= "ConsigneeCountry=$fromCountry&ShipperCountry=$toCountry&";
        
            // the weight of this package
            $this->queryString .= "PackageActualWeight=$weight&";

            // "ResidentialInd": 0 is commercial; 1 is residential; "PackagingType" is provided by shipper
            $this->queryString .= "ResidentialInd=$resOrInd&PackagingType=00";             
        }

        function getResponse()
        {
            $fp = @fopen( $this->getServer() . $this->getQueryString(), 'r'); 

            if( $fp == false )
                return false;

            while( $buffer = @fgets($fp, 4096) )
            {
                if ( strstr( $buffer, "UPSOnLine" ) )
                    $this->loadVals( $buffer );
            }

            @fclose( $fp );
            return true;
        }

    } // end of class

    // ********************************************************************************************
    // class FedExShipment extends Shipment:
    //          Class for Federal Express shipping prices and information.
    // ********************************************************************************************
    class FedExShipment extends Shipment
    {
        var $DOMESTIC_GROUND = "Ground";
        var $HOME_DELIVERY = "HomeD";

        var $domGnd;
        var $homeDel;
        var $svcType;
        var $zone;
        var $transitTime;
        var $parent;

        function FedExShipment( $gndOrHome )
        {
            $this->parent = new Shipment();

            $this->server = "http://grd.fedex.com/cgi-bin/rrr2010.exe?";

            $this->keys = array( "Zone",
                                 "TransitTime",  
                                 "TotalCharges" );

            $this->vals = array();

            $this->domGnd = 0;
            $this->homeDel = 0;
            $this->zone = 0;
            $this->transitTime = 0;
            $this->svcType = ( $gndOrHome == "0" ) ? "Ground" : "HomeD";
            $this->maxWeight = ( $gndOrHome == "0" ) ? 150 : 70;  
            $this->msg = "<br>Unable to connect to the Federal Express server at " . $this->server . "<br>";
        }

        function currentServiceType()
        {
            return $this->svcType;
        }        

        function getDomesticGroundPrice()
        {
            return sprintf( "%01.2f", $this->domGnd );
        }

        function getHomeDeliveryPrice()
        {
            return sprintf( "%01.2f", $this->homeDel );            
        }
        
        // *****
        // Set the status
        function setStatus()
        {
            if ( ( $this->domGnd > 0 ) || ( $this->homeDel > 0 ) )
                $this->retcode = true;
        }

        function loadVals( $valLine )
        {            
            $pos = 0;

            for ( $i = 0; $i < count($this->keys); $i++ )
            {
                $startStr = "<!" . $this->keys[$i] . ">";
                $endStr = "<!/" . $this->keys[$i] . ">";

                if ( strstr( $valLine, $startStr ) )
                {
                    $stripVal = trim( strip_tags( $valLine ) );

                    if ( $this->keys[$i] == "Zone" )
                        $this->zone = $stripVal;
                    else if ( $this->keys[$i] == "TransitTime" )
                        $this->transitTime = $stripVal;
                    else if ( $this->keys[$i] == "TotalCharges" )
                    {
                        if ( $this->currentServiceType() == $this->DOMESTIC_GROUND )
                            $this->domGnd += $stripVal;
                        if ( $this->currentServiceType() == $this->HOME_DELIVERY )
                            $this->homeDel += $stripVal;                     
                    }
               }
            }

            $this->setStatus();
            
        }        

        function buildQueryString( $toCode, $fromCode, $weight, $toCountry = "US", $fromCountry = "US" )
        {
            // need to pass the application information
            $this->queryString = "func=Rate&";

            // "Screen" specifies Domestic Ground ("Ground") or Home Delivery ("HomeD")
            $this->queryString .= "Screen=" . $this->currentServiceType() . "&";

            // the "To" and "From" ZIP codes
            $this->queryString .= "OriginZip=$fromCode&DestZip=$toCode&";
        
            // "To" and "From" countries
            $this->queryString .= "DestCountryCode=$fromCountry&OriginCountryCode=$toCountry&";
        
            // the weight of this package
            $this->queryString .= "Weight=$weight";
        }

        function getResponse()
        {        
            $fp = @fopen( $this->getServer() . $this->getQueryString(), 'r'); 
        
            if( $fp == false )
                return false;
        
            while( $buffer = @fgets($fp, 4096) )
            {
                if ( strstr( $buffer, "<!" ) )
                    if ( !strstr( $buffer, "<!--" ) )
                        $this->loadVals( $buffer );
            }

            @fclose( $fp );

            return true;
        }
    } // end of class


    // *****
    // Recursive function to make sure the weight of the given package is below the maximum
    function weighPackages( $curWeight, $maxWeight, $holder, $index )
    {
        if ( $curWeight > $maxWeight )
        {
            $holder[$index] = $maxWeight;
            $index++;

            $temp = $curWeight - $maxWeight;

            if( $temp > $maxWeight )
                weighPackages( $temp, $maxWeight, &$holder, &$index );
            else
                $holder[$index] = $temp;
        }
        else
            $holder[$index] = $curWeight;

        return true;
    }    
?>
