<?
/*
* phorSale - An eCommerce application for PHP and mySQL.
* Copyright (C) 2001 Michael Kmiec <mike@shifter.org>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/******
*
*   This is where the shipping prices will be displayed, based on the order item's 
*   "product_shipping_from" field and the order's "ship_postal_code" field.
*
******/

// register the order object, for persistance sake
session_start();

// standard include files
require ("common/config.inc.php");
include("common/common.functions.inc.php");
include("common/order.functions.inc.php");
include("common/shopper.functions.inc.php");

// shipping include files
include("common/shipment.inc.php");

// definitions
$sID = session_id();
$nextPage = "orderSummary.php";

    if ( isSet( $SMI ) )
    {
        // validate necessary information
        $errMsg = "";
        if ( !strlen( $ship_method ) && !strlen( $shipping_number ) )
            $errMsg .= "<li> You must select a Shipping Method or enter a Shipping Number.";
        else if ( strlen( $shipping_number ) && !strlen( $ship_num_type ) )
            $errMsg .= "<li> You must choose which Type of Shipping Number you entered.";
            

        if ( strlen( $errMsg ) == 0 )
        {        
            // save shipping information to order; give precedence to an entered shipping number
            if ( strlen( $shipping_number ) )
            {
                $order["ship_price"] = 0;
                switch ( $ship_num_type )
                {
                    case "UPSNUMBER":                        
                        $order["ship_method"] = "UPS Shipping Number";
                        $order["shipping_number"] = $shipping_number;
                        $order["ship_num_type"] = "UPSNUMBER";
                        break;
                    case "FDXNUMBER":                         
                        $order["ship_method"] = "FedEx Shipping Number";
                        $order["shipping_number"] = $shipping_number;
                        $order["ship_num_type"] = "FDXNUMBER";
                        break;
                }
            }
            else
            {
                $order["ship_num_type"] = "";
                $order["shipping_number"] = "";

                switch ( $ship_method )
                {
                    case "UPSGND":
                        $order["ship_method"] = "UPS Ground";
                        $order["ship_price"] = $UPSGND;
                        break;
                    case "UPS3DS":
                        $order["ship_method"] = "UPS Three Day Select";
                        $order["ship_price"] = $UPS3DS;
                        break;
                    case "UPS2DA":
                        $order["ship_method"] = "UPS Second Day Air";
                        $order["ship_price"] = $UPS2DA;
                        break;
                    case "UPSNDA":
                        $order["ship_method"] = "UPS Next Day Air";
                        $order["ship_price"] = $UPSNDA;
                        break;
                    case "FDXGND":
                        $order["ship_method"] = "FedEx Domestic Ground";
                        $order["ship_price"] = $FDXGND;
                        break;
                    case "FDXHDL":
                        $order["ship_method"] = "FedEx Home Delivery";
                        $order["ship_price"] = $FDXHDL;
                        break;                       
                }
            }

            // send them to the order summary page
            header("Location: " . $nextPage . "?eCommID=" . $shopperID . "&PHPSESSID=" . $sID);
        }
    }

    $shopper = getShopper( $shopperID );    

    // figure out the shipping prices
    $maxUPSWeight = 150;

    $ShipperPostalCode = $order["items"]["shippingFrom"];
    $ConsigneePostalCode = $order["ship_postal_code"];

    // If there is a Company provided in the Shipping Address, assume this is a 
    // business/industrial shipment.
    $deliveryType = ( strlen( trim( $order["ship_company"] ) ) > 0 ) ? "0" : "1";

    for ( $ind = 0; $ind < count( $order["items"] ); $ind++ )
    {
        $unitWeight = $order["items"][$ind]["weight"];
        $unitPer = $order["items"][$ind]["units"];
        $numUnits = $order["items"][$ind]["quantity"];
        $unitBreak = $numUnits / $unitPer;
        $totWeight += $unitBreak * $unitWeight;
    }
    
    $upsPkgWeights = array();
    $fedexPkgWeights = array();    
    
    // construct a new collection for the data
    $s = new UPSShipment();
    $fs = new FedExShipment( $deliveryType );

    // need to get the total weight of order, and split packages appropriately
    $idx = 0;
    weighPackages( $totWeight, $s->maxWeight, &$upsPkgWeights, &$idx );
    // reset the index, then go again
    $idx = 0;
    weighPackages( $totWeight, $fs->maxWeight, &$fedexPkgWeights, &$idx );

    // go through UPS packages
    for ( $x=0; $x<count( $upsPkgWeights ); $x++ )
    {                
        $s->buildQueryString( $ConsigneePostalCode, $ShipperPostalCode, $upsPkgWeights[$x], $deliveryType );
        $s->getResponse();
    }

    // go through FedEx packages
    for ( $x=0; $x<count( $fedexPkgWeights ); $x++ )
    {
        $fs->buildQueryString( $ConsigneePostalCode, $ShipperPostalCode, $fedexPkgWeights[$x] );
        $fs->getResponse();     
    }  

    $shipPrices = "";
    $shipError = "";

    // build shipping select list
    $shipSelect = "<select size=\"1\" name=\"ship_method\" onChange=\"switchShip();\">\n";
    $shipSelect .= "<option value=\"\"";
    if ( $order[ship_method] == "" ) $shipSelect .= " SELECTED";
    $shipSelect .= "> Select Shipping Method";    
    if ( $s->success() )
    {
        if ( $s->getGroundPrice() > 0 )
        {
            $shipSelect .= "<option value=\"UPSGND\"";
            if ( $order[ship_method] == "UPS Ground" ) $shipSelect .= " SELECTED";        
            $shipSelect .= "> UPS Ground for $" . $s->getGroundPrice();
        }

        if ( $s->getThreeDaySelectPrice() > 0 )
        {
            $shipSelect .= "<option value=\"UPS3DS\"";
            if ( $order[ship_method] == "UPS Three Day Select" ) $shipSelect .= " SELECTED";
            $shipSelect .= "> UPS Three Day Select for $" . $s->getThreeDaySelectPrice();
        }

        if ( $s->getSecondDayAirPrice() > 0 )
        {
            $shipSelect .= "<option value=\"UPS2DA\"";
            if ( $order[ship_method] == "UPS Second Day Air" ) $shipSelect .= " SELECTED";
            $shipSelect .= "> UPS Second Day Air for $" . $s->getSecondDayAirPrice();
        }

        if ( $s->getNextDayAirPrice() > 0 )
        {
            $shipSelect .= "<option value=\"UPSNDA\"";
            if ( $order[ship_method] == "UPS Next Day Air" ) $shipSelect .= " SELECTED";
            $shipSelect .= "> UPS Next Day Air for $" . $s->getNextDayAirPrice();
        }        

        $shipPrices .= "<input type=\"hidden\" name=\"UPSGND\" value=\"" . $s->getGroundPrice() . "\">\n";
        $shipPrices .= "<input type=\"hidden\" name=\"UPS3DS\" value=\"" . $s->getThreeDaySelectPrice() . "\">\n";
        $shipPrices .= "<input type=\"hidden\" name=\"UPS2DA\" value=\"" . $s->getSecondDayAirPrice() . "\">\n";
        $shipPrices .= "<input type=\"hidden\" name=\"UPSNDA\" value=\"" . $s->getNextDayAirPrice() . "\">\n";
    }
    else
        $shipError .= $s->getMessage();

    if ( $fs->success() )
    {
        if ( $fs->getDomesticGroundPrice() > 0 )
        {
            $shipSelect .= "<option value=\"FDXGND\"";
            if ( $order[ship_method] == "FedEx Domestic Ground" ) $shipSelect .= " SELECTED";        
            $shipSelect .= "> Federal Express Domestic Ground for $" . $fs->getDomesticGroundPrice();
        }

        if ( $fs->getHomeDeliveryPrice() > 0 )
        {
            $shipSelect .= "<option value=\"FDXHDL\"";
            if ( $order[ship_method] == "FedEx Home Delivery" ) $shipSelect .= " SELECTED";
            $shipSelect .= "> Federal Express Home Delivery for $" . $fs->getHomeDeliveryPrice();
        }

        $shipPrices .= "<input type=\"hidden\" name=\"FDXGND\" value=\"" . $fs->getDomesticGroundPrice() . "\">\n";
        $shipPrices .= "<input type=\"hidden\" name=\"FDXHDL\" value=\"" . $fs->getHomeDeliveryPrice() . "\">\n";
    }
    else
        $shipError .= $fs->getMessage();

    $shipSelect .= "</select>";

    $clientSideScript = "
    <script language=\"JavaScript\">
    <!--
    function switchShip()
    {
        document.shipMethod.shipping_number.value = \"\";
    }
    // -->
    </script>";

    $title = "Shipping Method";
    require 'common/header.inc.php';
?>

<font face="<?echo $fontTag;?>" size="+2" color="#000000">Shipping Method</font><hr size="1">

<form name="shipMethod" action="<?echo basename($PHP_SELF);?>" method="POST">
<input type="hidden" name="eCommID" value="<?echo $shopperID;?>">
<input type="hidden" name="PHPSESSID" value="<?echo $sID;?>">
<input type="hidden" NAME="redir" VALUE="<?echo $nextPage;?>">
<input type="hidden" NAME="SMI" VALUE="true">
<? echo $shipPrices; ?>

<table border="0" cellspacing="1" width="80%">

<?
if ( strlen( $errMsg ) ) 
{
?>
    <tr>
        <td colspan="2" align="left">
            <font face="<?echo $fontTag; ?>" size="2" color="#CC0000"><b>
                There appears to be a problem...
            <ul><?echo $errMsg;?></ul>
            </b></font><p>
        </td>
    </tr>
<?
}
?>
    <tr>
        <td align="left" colspan="2"><font face="<?echo $fontTag;?>" size="2">
            Please select your preferred Shipping Method. If you choose one of the listed methods, the price will added to 
            your order. If you choose to use your own Shipping Number, your account will be billed instead.
            <p>            
            All information in <font color="green"><b>green</b></font> is required.</font>
        </td>
    </tr>
    <tr>
        <td align="center" colspan="2">&nbsp;</td>        
    </tr>
    <tr>
        <td align="right"><font face="<?echo $fontTag;?>" size="2" color="green"><b>Shipping Method</b></font></td>
        <td>
            <? echo $shipSelect; ?>
        </td>
    </tr>
<?  if ( strlen( $shipError ) )
    {
?>
    <tr>
        <td align="center" colspan="2">
            <font face="<?echo $fontTag;?>" color="red" size="-2"><b><? echo $shipError; ?></b></font>
        </td>
    </tr>
<?
    }
?>  
    <tr>
        <td align="center" colspan="2">&nbsp;</td>    
    </tr>
    <tr>
        <td colspan="2" align="center"><font face="<?echo $fontTag;?>" size="+2"><p><b>Or...</b><p></font></td>
    </tr>
    <!-- tr>
        <td align="center" colspan="2">&nbsp;</td>    
    </tr -->    
    <tr>
        <td align="right"><font face="<?echo $fontTag;?>" size="2" color="green"><b>Enter Your Shipping Number</b></font></td>
        <td>            
            <input type="text" maxlength="100" name="shipping_number" size="30" value="<? echo $order[shipping_number]; ?>">
        </td>
    </tr>
    <tr>
        <td align="right"><font face="<?echo $fontTag;?>" size="2" color="green"><b>This is a...</font></td>
        <td>
            <font face="<?echo $fontTag;?>" size="2">
                <input type="radio" name="ship_num_type" <? if ( $order[ship_num_type] == "UPSNUMBER" ) echo "checked"; ?> value="UPSNUMBER"> UPS Shipping Number
                <input type="radio" name="ship_num_type" <? if ( $order[ship_num_type] == "FDXNUMBER" ) echo "checked"; ?> value="FDXNUMBER">  Federal Express Shipping Number
            </font>
        </td>
    </tr>
     <tr>
        <td align="center" colspan="2">&nbsp;</td>    
    </tr>
    <tr>
        <td width="100%" align="center" colspan="2">
            <input type="submit" value="Next Step"><input type="reset" value="Reset">
        </td>
    </tr>
</table>
</form>
<?
    if ($order["status"]="ready")
        $order["status"]="check";
?>

<? require 'common/footer.inc.php'; ?>
